// Full code for the prosthetic hand. The servo motors position is determined by values read on an X-Y joystick (Y-plane).  
// The joystick also turns the motors on and off, and toggles between pinch and power grip patterns (X-plane). Haptic 
// motors vibrate when a fingertip pressure sensor is pressed. Threshold values determine which one vibrates, indicating
// to the user the level of force on the fingertip. One of 4 haptic motors vibrates in one of the 123 preset vibration
// patterns found in the datasheet for the Sparkfun DRV2605L  haptic driver.

#include <Sparkfun_DRV2605L.h> //SparkFun Haptic Motor Driver Library 
#include <Wire.h> //I2C library 
#include <Servo.h> //Servo library
Servo lockServo;
Servo servoL;
Servo servoR;
SFE_HMD_DRV2605L HMD; //Create haptic motor driver object

const int analogOutPin = 8; // Analog output pin that the Haptic Motor Driver is attached to
int VIB1 = 4; // vibration motor 1 pin
int VIB2 = 5; // vibration motor 2 pin
int VIB3 = 6; // vibration motor 3 pin
int VIB4 = 7; // vibration motor 4 pin
int togSwitch = 1;
int gripSwitch = 1;
int posR = 0; // set right servo position (fingers open)
int posL = 180; // set left servo position (fingers open)
int joyX;
int joyY;


void setup()  {
  pinMode(VIB1, OUTPUT);
  pinMode(VIB2, OUTPUT);
  pinMode(VIB3, OUTPUT);
  pinMode(VIB4, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(11, OUTPUT);
  HMD.begin();
  Serial.begin(9600);
  HMD.Mode(0); // Internal trigger input mode -- Must use the GO() function to trigger playback.
  HMD.MotorSelect(0x36); // ERM motor, 4x Braking, Medium loop gain, 1.365x back EMF gain
  HMD.Library(2); //1-5 & 7 for ERM motors, 6 for LRA motors
  servoL.attach(12);
  servoL.write(posL);
  servoR.attach(11);
  servoR.write(posR);
}

void loop() {

  int joyY = analogRead(A0); // read joystick Y axis value
  int joyX = analogRead(A1); // read joystick X axis value
  int pressureSensor  = analogRead(A2); // read pressure sensor value

  // pushing joystick in thumb extension toggles servo control ON and OFF
  if (joyX > 650) {
    delay(500); // delay of 0.5s for debouncing and to avoid accidental double clicks
    togSwitch = togSwitch * -1;

    // alert user to change in state using vibration
    digitalWrite(VIB1, HIGH);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, LOW);
    HMD.Waveform(1, 64); // second value corresponds to a preset vibration pattern 1-123
    HMD.go();
    delay(500); // delay for motor pattern to complete before disconnecting motors
    digitalWrite(VIB1, LOW);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, LOW);
  }

  if (togSwitch == 1) {

    // pushing joystick in thumb flexion toggles between pinch and power grip patterns
    if (joyX < 250) {
      delay(500); // delay of 0.5s for debouncing and to avoid accidental double clicks
      gripSwitch = gripSwitch * -1;
      if (gripSwitch == -1) {
        servoR.write(180); // set last 3 fingers to fully closed until gripswitch is triggered again.
      }
      else
        servoR.write(180 - posR); // reset last 3 fingers to match current index and thumb positions

      // alert user to change in state using vibration
      digitalWrite(VIB1, HIGH);
      digitalWrite(VIB2, LOW);
      digitalWrite(VIB3, LOW);
      digitalWrite(VIB4, LOW);
      HMD.Waveform(1, 64); // second value corresponds to a preset vibration pattern 1-123
      HMD.go();
      delay(500); // delay for motor pattern to complete before disconnecting motors
      digitalWrite(VIB1, LOW);
      digitalWrite(VIB2, LOW);
      digitalWrite(VIB3, LOW);
      digitalWrite(VIB4, LOW);
    }

    // servo control 

    if (joyY < 400) { // if joystick moves above this value, close fingers
      if (gripSwitch == 1) {
        servoR.write(posR += 1);
      }
      servoL.write(posL -= 1);
    }

    else if (joyY > 580) { // if joystick moves above this value, open fingers
      if (gripSwitch == 1) {
        servoR.write(posR -= 1);
      }
      servoL.write(posL += 1 );
    }

    // constrain pos value to servo limits
    if (posR >= 180) {
      posR = 180;
    }
    else if (posR <= 0) {
      posR = 0;
    }
    if (posL >= 180) {
      posL = 180;
    }
    else if (posL <= 0) {
      posL = 0;
    }
  }
  //END**** Servo Control

  //BEGIN**** Haptic code that vibrates different motors depending on the value read from the pressure sensor

  // drive motor 1
  if ( 0 < pressureSensor && pressureSensor <= 125 ) {
    digitalWrite(VIB1, HIGH);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, LOW);
    HMD.Waveform(1, 47); // second value corresponds to a preset vibration pattern 1-123
    HMD.go();
    delay(10);
  }

  // drive motor 2
  else if ( 125 < pressureSensor && pressureSensor <= 250  ) {
    digitalWrite(VIB1, LOW);
    digitalWrite(VIB2, HIGH);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, LOW);
    HMD.Waveform(1, 47); // second value corresponds to a preset vibration pattern 1-123
    HMD.go();
    delay(10);
  }

  // drive motor 3
  else if ( 250 < pressureSensor && pressureSensor <= 375) {
    digitalWrite(VIB1, LOW);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, HIGH);
    digitalWrite(VIB4, LOW);
    HMD.Waveform(1, 47); // second value corresponds to a preset vibration pattern 1-123
    HMD.go();
    delay(10);
  }
  // drive motor 4
  else if ( pressureSensor > 375) {
    digitalWrite(VIB1, LOW);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, HIGH);
    HMD.Waveform(1, 47); // second value corresponds to a preset vibration pattern 1-123
    HMD.go();
    delay(10);
  }

  // no motors driven
  else {
    digitalWrite(VIB1, LOW);
    digitalWrite(VIB2, LOW);
    digitalWrite(VIB3, LOW);
    digitalWrite(VIB4, LOW);
    delay(10);
  }

  //END**** Haptic Code

  // print sensor values
  Serial.print("   X value = ");
  Serial.print(joyX);
  Serial.print("   Y value = ");
  Serial.print(joyY);
  Serial.print("   Servo position: ");
  Serial.println(posR);
  Serial.print("   Fingertip Sensor:  ");
  Serial.println(pressureSensor);
}
